package leo.tbd.vc

import java.net.URI

private fun execRuntime(vararg args: String) =
    Runtime.getRuntime().exec(args)

private enum class OS(
    val openMethod: (URI) -> Any,
    val alt: String? = null,
) {
    WIN({
        try {
            val desktopClass = Class.forName("java.awt.Desktop")
            val getDesktopMethod = desktopClass.getMethod("getDesktop")
            val desktopInstance = getDesktopMethod.invoke(null)
            val browseMethod = desktopClass.getMethod("browse", URI::class.java)
            browseMethod.invoke(desktopInstance, it)
        } catch (e: Exception) {
            execRuntime(
                "rundll32",
                "url.dll,FileProtocolHandler",
                "$it"
            )
        }
    }),
    MAC({ execRuntime("open", "$it") }),
    NIX({ execRuntime("xdg-open", "$it") }, "nux")
}

private val os = System.getProperty("os.name").lowercase().let { os ->
    OS.entries.first { os.contains(it.name.lowercase()) || it.alt?.let { it1 -> os.contains(it1) } ?: false }
}

fun openUri(uri: URI) {
    os.openMethod(uri)
}
